@@ -8,11 +8,10 @@ import re
 from distutils.spawn import find_executable
 
 from andes.utils.misc import is_notebook
-from andes.core.var import Algeb, State
-from andes.main import config_logger
+from andes.core.var import BaseVar, Algeb, ExtAlgeb
+from andes.utils.paths import get_dot_andes_path
 from andes.shared import np, mpl, plt
 
-config_logger(log_file=None)
 logger = logging.getLogger(__name__)
 
 
@@ -51,8 +50,6 @@ class TDSData(object):
         else:
             raise NotImplementedError(f'Unknown mode {self._mode}.')
 
-        self._latex_warn = True
-
     def _process_names(self):
         self.file_name, self.file_ext = os.path.splitext(self.full_name)
         self._npy_file = os.path.join(self._path, self.file_name + '.npy')
@@ -144,7 +141,7 @@ class TDSData(object):
 
     def load_npy_or_csv(self, delimiter=','):
         """
-        Load the npy or csv file into internal data structures `self._data`.
+        Load the npy or csv file into internal data structures `self._xy`.
 
         Parameters
         ----------
@@ -239,14 +236,14 @@ class TDSData(object):
             fd.write(','.join(header) + '\n')
             np.savetxt(fd, body, fmt=fmt, delimiter=',')
 
-        logger.info(f'CSV data saved in <{path}>.')
+        logger.info(f'CSV data saved to "{path}".')
 
     def plot(self, yidx, xidx=(0,), a=None, ycalc=None,
              left=None, right=None, ymin=None, ymax=None, ytimes=None,
              xlabel=None, ylabel=None,
-             legend=True, grid=False, greyscale=False,
-             latex=True, dpi=150, savefig=None, save_format=None, show=True,
-             use_bqplot=False, latex_warn=True, **kwargs):
+             legend=None, grid=False, greyscale=False,
+             latex=True, dpi=150, line_width=1.0, font_size=12, savefig=None, save_format=None, show=True,
+             title=None, use_bqplot=False, **kwargs):
         """
         Entery function for plot scripting. This function retrieves the x and y values based
         on the `xidx` and `yidx` inputs and then calls `plot_data()` to do the actual plotting.
@@ -269,15 +266,18 @@ class TDSData(object):
             Figure and axis handles
         """
         if self._mode == 'memory':
-            if isinstance(yidx, (State, Algeb)):
+            if isinstance(yidx, BaseVar):
+                if yidx.n == 0:
+                    logger.error(f"Variable <{yidx.name}> contains no values.")
+                    return
                 offs = 1
-                if isinstance(yidx, Algeb):
+                if isinstance(yidx, (Algeb, ExtAlgeb)):
                     offs += self.dae.n
 
-                if a is None:
-                    yidx = yidx.a + offs
-                else:
-                    yidx = np.take(yidx.a, a) + offs
+                yidx = yidx.a + offs
+
+        if a is not None:
+            yidx = np.take(yidx, a)
 
         x_value = self.get_values(xidx)
         y_value = self.get_values(yidx)
@@ -306,16 +306,16 @@ class TDSData(object):
             return self.bqplot_data(xdata=x_value, ydata=y_value, xheader=x_header, yheader=y_header,
                                     left=left, right=right, ymin=ymin, ymax=ymax,
                                     xlabel=xlabel, ylabel=ylabel, legend=legend, grid=grid, greyscale=greyscale,
-                                    latex=latex, dpi=dpi, savefig=savefig, save_format=save_format, show=show,
-                                    latex_warn=latex_warn,
+                                    latex=latex, dpi=dpi, line_width=line_width, font_size=font_size,
+                                    savefig=savefig, save_format=save_format, show=show, title=title,
                                     **kwargs)
 
         else:
             return self.plot_data(xdata=x_value, ydata=y_value, xheader=x_header, yheader=y_header,
                                   left=left, right=right, ymin=ymin, ymax=ymax,
                                   xlabel=xlabel, ylabel=ylabel, legend=legend, grid=grid, greyscale=greyscale,
-                                  latex=latex, dpi=dpi, savefig=savefig, save_format=save_format, show=show,
-                                  latex_warn=latex_warn,
+                                  latex=latex, dpi=dpi, line_width=line_width, font_size=font_size,
+                                  savefig=savefig, save_format=save_format, show=show, title=title,
                                   **kwargs)
 
     def data_to_df(self):
@@ -330,7 +330,8 @@ class TDSData(object):
 
     def bqplot_data(self, xdata, ydata, xheader=None, yheader=None, xlabel=None, ylabel=None,
                     left=None, right=None, ymin=None, ymax=None, legend=True, grid=False, fig=None,
-                    latex=True, dpi=150, greyscale=False, savefig=None, save_format=None, show=True, **kwargs):
+                    latex=True, dpi=150, line_width=1.0, greyscale=False, savefig=None, save_format=None,
+                    show=True, **kwargs):
         """
         Plot with ``bqplot``. Experimental and imcomplete.
         """
@@ -344,7 +345,7 @@ class TDSData(object):
 
         plt.figure(dpi=dpi)
         plt.plot(xdata, ydata.transpose(),
-                 linewidth=1.5,
+                 linewidth=line_width,
                  )
 
         if yheader:
@@ -353,9 +354,9 @@ class TDSData(object):
         plt.show()
 
     def plot_data(self, xdata, ydata, xheader=None, yheader=None, xlabel=None, ylabel=None, line_styles=None,
-                  left=None, right=None, ymin=None, ymax=None, legend=True, grid=False, fig=None, ax=None,
-                  latex=True, dpi=150, greyscale=False, savefig=None, save_format=None, show=True,
-                  latex_warn=True, **kwargs):
+                  left=None, right=None, ymin=None, ymax=None, legend=None, grid=False, fig=None, ax=None,
+                  latex=True, dpi=150, line_width=1.0, font_size=12, greyscale=False, savefig=None,
+                  save_format=None, show=True, title=None, **kwargs):
         """
         Plot lines for the supplied data and options. This functions takes `xdata` and `ydata` values. If
         you provide variable indices instead of values, use `plot()`.
@@ -410,6 +411,10 @@ class TDSData(object):
             File extension string (pdf, png or jpg) for the savefig format
         dpi : int
             Dots per inch for screen print or save. savefig uses a minimum of 200 dpi
+        line_width : float
+            Plot line width
+        font_size : float
+            Text font size (labels and legends)
         show : bool
             True to show the image
         kwargs
@@ -420,8 +425,7 @@ class TDSData(object):
         (fig, ax)
             The figure and axis handles
         """
-        mpl.rc('font', family='Arial', size=12)
-        self._latex_warn = latex_warn
+        mpl.rc('font', family='Arial', size=font_size)
 
         if not isinstance(ydata, np.ndarray):
             TypeError("ydata must be a numpy array. Retrieve with get_values().")
@@ -431,12 +435,7 @@ class TDSData(object):
 
         n_lines = ydata.shape[1]
 
-        using_latex = set_latex(latex)
-        if using_latex and self._latex_warn:
-            logger.info('Using LaTeX for rendering. If an error occurs:')
-            logger.info('a) If you are using `andes plot`, disable with optino "-d",')
-            logger.info('b) If you are using `plot()`, set "latex=False".')
-            self._latex_warn = False
+        set_latex(latex)
 
         # set default x min based on simulation time
         if not left:
@@ -462,7 +461,7 @@ class TDSData(object):
                     ydata[:, i],
                     ls=line_styles[i],
                     label=yheader[i] if yheader else None,
-                    linewidth=1,
+                    linewidth=line_width,
                     color='0.2' if greyscale else None,
                     )
 
@@ -485,9 +484,17 @@ class TDSData(object):
         if grid:
             ax.grid(b=True, linestyle='--')
 
+        if yheader is None:
+            legend = False
+        elif legend is None:
+            if len(yheader) <= 8:
+                legend = True
+
         if legend:
-            if yheader:
-                ax.legend()
+            ax.legend()
+
+        if title:
+            ax.set_title(title)
 
         plt.draw()
 
@@ -567,7 +574,7 @@ def parse_y(y, upper, lower=0):
             try:
                 y[idx] = int(item)
             except ValueError:
-                logger.warning('y contains non-numerical values <{}>. Parsing could not proceed.'.format(item))
+                logger.warning(f'y contains non-numerical values <{item}>. Parsing cannot proceed.')
                 return []
 
         y_from_range = list(range(*y))
@@ -772,9 +779,25 @@ def set_latex(enable=True):
     """
 
     has_dvipng = find_executable('dvipng')
+
     if has_dvipng and enable:
         mpl.rc('text', usetex=True)
+
+        no_warn_file = os.path.join(get_dot_andes_path(), '.no_warn_latex')
+        if not os.path.isfile(no_warn_file):
+            logger.info('Info:')
+            logger.info('Using LaTeX for rendering. If an error occurs:')
+            logger.info('a) If you are using `andes plot`, disable with option "-d",')
+            logger.info('b) If you are using `plot()`, set "latex=False".')
+
+            try:
+                with open(os.path.join(get_dot_andes_path(), '.no_warn_latex'), 'w'):
+                    pass
+            except OSError:
+                pass
+
         return True
+
     else:
         mpl.rc('text', usetex=False)
         return False
