@@ -20,6 +20,8 @@ class PFlow(BaseRoutine):
                                      ('method', 'NR'),
                                      ('n_factorize', 4),
                                      ('report', 1),
+                                     ('degree', 0),
+                                     ('init_tds', 0),
                                      )))
         self.config.add_extra("_help",
                               tol="convergence tolerance",
@@ -27,6 +29,8 @@ class PFlow(BaseRoutine):
                               method="calculation method",
                               n_factorize="first N iterations to factorize Jacobian in dishonest method",
                               report="write output report",
+                              degree='use degree in report',
+                              init_tds="initialize TDS after PFlow",
                               )
         self.config.add_extra("_alt",
                               tol="float",
@@ -34,21 +38,25 @@ class PFlow(BaseRoutine):
                               max_iter=">=10",
                               n_factorize=">0",
                               report=(0, 1),
+                              degree=(0, 1),
+                              init_tds=(0, 1),
                               )
-        self.models = system.find_models('pflow')
 
         self.converged = False
         self.inc = None
         self.A = None
         self.niter = None
         self.mis = []
+        self.models = OrderedDict()
 
     def init(self):
+        self.models = self.system.find_models('pflow')
         self.converged = False
         self.inc = None
         self.A = None
         self.niter = None
         self.mis = []
+
         self.system.init(self.models)
         logger.info('Power flow initialized.')
         return self.system.dae.xy
@@ -63,20 +71,20 @@ class PFlow(BaseRoutine):
             maximum absolute mismatch
         """
         system = self.system
-        # evaluate discrete, differential, algebraic, and jacobians
-        system.e_clear()
-        system.l_update_var()
-        system.f_update()
-        system.g_update()
-        system.l_check_eq()
-        system.l_set_eq()
+        # evaluate discrete, differential, algebraic, and Jacobians
+        system.dae.clear_fg()
+        system.l_update_var(self.models)
+        system.s_update_var(self.models)
+        system.f_update(self.models)
+        system.g_update(self.models)
+        system.l_update_eq(self.models)
         system.fg_to_dae()
 
         if self.config.method == 'NR':
-            system.j_update()
+            system.j_update(models=self.models)
         elif self.config.method == 'dishonest':
             if self.niter < self.config.n_factorize:
-                system.j_update()
+                system.j_update(self.models)
 
         # prepare and solve linear equations
         self.inc = -matrix([matrix(system.dae.f),
@@ -104,6 +112,7 @@ class PFlow(BaseRoutine):
         out = list()
         out.append('')
         out.append('-> Power flow calculation')
+        out.append(f'Sparse Solver: {self.solver.sparselib.upper()}')
         out.append(f'Method: {self.config.method} method')
         out_str = '\n'.join(out)
         logger.info(out_str)
@@ -122,6 +131,7 @@ class PFlow(BaseRoutine):
         self.init()
         if system.dae.m == 0:
             logger.error("Loaded case contains no power flow element.")
+            system.exit_code = 1
             return False
 
         t0, _ = elapsed()
@@ -136,7 +146,7 @@ class PFlow(BaseRoutine):
             elif self.niter > self.config.max_iter:
                 break
             elif np.isnan(mis).any():
-                logger.error(f'NaN found in solution. Convergence not likely')
+                logger.error('NaN found in solution. Convergence not likely')
                 self.niter = self.config.max_iter + 1
                 break
             elif mis > 1e4 * self.mis[0]:
@@ -157,9 +167,12 @@ class PFlow(BaseRoutine):
 
         else:
             logger.info(f'Converged in {self.niter+1} iterations in {s1}.')
+            if self.config.init_tds:
+                system.TDS.init()
             if self.config.report:
                 system.PFlow.report()
 
+        system.exit_code = 0 if self.converged else 1
         return self.converged
 
     def report(self):
@@ -186,14 +199,14 @@ class PFlow(BaseRoutine):
         system.dae.x = xy[:system.dae.n]
         system.dae.y = xy[system.dae.n:]
         system.vars_to_models()
-        system.e_clear()
 
+        system.dae.clear_fg()
         system.l_update_var()
         system.f_update()
         system.g_update()
-        system.l_check_eq()
-        system.l_set_eq()
+        system.l_update_eq()
         system.fg_to_dae()
+
         return system.dae.fg
 
     def newton_krylov(self, verbose=False):
@@ -213,7 +226,7 @@ class PFlow(BaseRoutine):
 
         """
         system = self.system
-        system.init()
+        system.init(system.exist.pflow)
         v0 = system.dae.xy
         try:
             ret = newton_krylov(self._fg_wrapper, v0, verbose=verbose)
