@@ -10,12 +10,13 @@ import inspect
 from collections import OrderedDict
 from typing import List, Dict, Tuple, Union, Optional
 
+from andes import __version__
 from andes.models import non_jit
 from andes.variables import FileMan, DAE
 from andes.routines import all_routines
 from andes.utils.tab import Tab
 from andes.utils.paths import get_config_path, get_pkl_path, confirm_overwrite
-from andes.core import Config, BaseParam, Model, ExtVar, AntiWindup
+from andes.core import Config, BaseParam, Model, AntiWindup
 
 from andes.shared import np, spmatrix, jac_names
 logger = logging.getLogger(__name__)
@@ -26,6 +27,16 @@ else:
     IP_ADD = False
 
 
+class ExistingModels(object):
+    """
+    Storage class for existing models
+    """
+    def __init__(self):
+        self.pflow = OrderedDict()
+        self.tds = OrderedDict()   # if a model needs to be initialized before TDS, set `flags.tds = True`
+        self.pflow_tds = OrderedDict()
+
+
 class System(object):
     """
     System contains models and routines for modeling and simulation.
@@ -48,7 +59,7 @@ class System(object):
         Numerical DAE storage
     files : andes.variables.fileman.FileMan
         File path storage
-    config : andes.core.config.Config
+    config : andes.core.Config
         System config storage
     models : OrderedDict
         model name and instance pairs
@@ -75,6 +86,8 @@ class System(object):
         self.groups = OrderedDict()        # group names and instances
         self.routines = OrderedDict()      # routine names and instances
         self.switch_times = np.array([])   # an array of ordered event switching times
+        self.n_switches = 0                # number of elements in `self.switch_times`
+        self.exit_code = 0                 # command-line exit code, 0 - normal, others - error.
 
         # get and load default config file
         self._config_path = get_config_path()
@@ -89,6 +102,8 @@ class System(object):
                                      ('mva', 100),
                                      ('store_z', 0),
                                      ('ipadd', 1),
+                                     ('warn_limits', 1),
+                                     ('warn_abnormal', 1),
                                      )))
 
         self.config.add_extra("_help",
@@ -96,14 +111,19 @@ class System(object):
                               mva='system base MVA',
                               store_z='store limiter status in TDS output',
                               ipadd='Use spmatrix.ipadd if available',
+                              warn_limits='warn variables initialized at limits',
+                              warn_abnormal='warn initialization out of normal values',
                               )
         self.config.add_extra("_alt",
                               freq="float",
                               mva="float",
                               store_z=(0, 1),
                               ipadd=(0, 1),
+                              warn_limits=(0, 1),
+                              warn_abnormal=(0, 1),
                               )
         self.config.check()
+        self.exist = ExistingModels()
 
         self.files = FileMan(case=case, **self.options)    # file path manager
         self.dae = DAE(system=self)                        # numerical DAE storage
@@ -113,23 +133,23 @@ class System(object):
         self.import_models()
         self.import_routines()  # routine imports come after models
 
-        self._models_flag = {'pflow': self.find_models('pflow'),
-                             'tds': self.find_models('tds'),
-                             'pflow_tds': self.find_models(('tds', 'pflow')),
-                             }
-
+        self._getters = dict(f=list(), g=list(), x=list(), y=list())
         self._adders = dict(f=list(), g=list(), x=list(), y=list())
         self._setters = dict(f=list(), g=list(), x=list(), y=list())
         self.antiwindups = list()
 
+        # internal flags
+        self.is_setup = False              # if system has been setup
+
     def _clear_adder_setter(self):
         """
         Clear adders and setters storage
         """
+        self._getters = dict(f=list(), g=list(), x=list(), y=list())
         self._adders = dict(f=list(), g=list(), x=list(), y=list())
         self._setters = dict(f=list(), g=list(), x=list(), y=list())
 
-    def prepare(self, quick=False):
+    def prepare(self, quick=False, incremental=False):
         """
         Generate numerical functions from symbolically defined models.
 
@@ -139,6 +159,13 @@ class System(object):
         ----------
         quick : bool, optional
             True to skip pretty-print generation to reduce code generation time.
+        incremental : bool, optional
+            True to generate only for modified models, incrementally.
+
+        Notes
+        -----
+        Option ``incremental`` compares the md5 checksum of all var and
+        service strings, and only regenerate for updated models.
 
         Examples
         --------
@@ -159,13 +186,67 @@ class System(object):
         Generated lambda functions will be serialized to file, but pretty prints (SymPy objects) can only exist in
         the System instance on which prepare is called.
         """
-        self._generate_symbols()
-        self._generate_equations()
-        self._generate_jacobians()
-        self._generate_initializers()
-        if quick is False:
-            self._generate_pretty_print()
+        import math
+
+        # consistency check for group parameters and variables
         self._check_group_common()
+
+        loaded_calls = self._load_pkl()
+
+        if loaded_calls is None:
+            incremental = False
+            logger.debug('calls.pkl does not exist. Incremental codegen disabled.')
+
+        if quick is False and incremental is True:
+            incremental = False
+            logger.debug('Incremental codegen is only allowed in quick mode.')
+
+        total = len(self.models)
+        width = math.ceil(math.log(total, 10))
+        for idx, (name, model) in enumerate(self.models.items()):
+            if incremental and \
+                    name in loaded_calls and \
+                    hasattr(loaded_calls[name], 'md5'):
+                if loaded_calls[name].md5 == model.get_md5():
+                    model.calls = loaded_calls[name]
+                    print(f"\r\x1b[K Code generation skipped for {name} ({idx + 1:>{width}}/{total:>{width}}).",
+                          end='\r', flush=True)
+                    continue
+
+            print(f"\r\x1b[K Code generation for {name} ({idx+1:>{width}}/{total:>{width}}).",
+                  end='\r', flush=True)
+            model.prepare(quick=quick)
+
+        self._store_calls()
+        self.dill()
+
+    def _prepare_mp(self, quick=False):
+        """
+        Code generation with multiprocessing. NOT WORKING NOW.
+
+        Warnings
+        --------
+        Function is not working. Serialization failed for `conj`.
+        """
+        from andes.shared import Pool
+        import dill
+        dill.settings['recurse'] = True
+
+        # consistency check for group parameters and variables
+        self._check_group_common()
+
+        def _prep_model(model: Model):
+            model.prepare(quick=quick)
+            return model
+
+        model_list = list(self.models.values())
+
+        # TODO: failed when serializing.
+        ret = Pool().map(_prep_model, model_list)
+
+        for idx, name in enumerate(self.models.keys()):
+            self.models[name] = ret[idx]
+
         self._store_calls()
         self.dill()
 
@@ -175,17 +256,38 @@ class System(object):
 
         This function is to be called after adding all device data.
         """
+        if self.is_setup:
+            logger.warning('System has been setup. Calling setup twice is not allowed.')
+            return
+
         self.collect_ref()
         self._list2array()     # `list2array` must come before `link_ext_param`
         self.link_ext_param()
         self.find_devices()    # find or add required devices
+        # no device addition or removal after this point
         self.calc_pu_coeff()
 
+        # store models with routine flags
+        self.store_existing()
+
         # assign address at the end before adding devices and processing parameters
-        self.set_address()
-        self.set_dae_names()
-        self.store_sparse_pattern()
-        self.store_adder_setter()
+        self.set_address(self.exist.pflow)
+        self.set_dae_names(self.exist.pflow)
+        self.store_sparse_pattern(self.exist.pflow)
+        self.store_adder_setter(self.exist.pflow)
+
+        self.is_setup = True
+
+    def store_existing(self):
+        """
+        Store existing models in `System.existing`.
+
+        TODO: Models with `TimerParam` will need to be stored anyway.
+        This will allow adding switches on the fly.
+        """
+        self.exist.pflow = self.find_models('pflow')
+        self.exist.tds = self.find_models('tds')
+        self.exist.pflow_tds = self.find_models(('tds', 'pflow'))
 
     def reset(self):
         """
@@ -199,9 +301,10 @@ class System(object):
             logger.error('Reset failed because TDS is initialized. \nPlease reload the test case to start over.')
             return
         self.dae.reset()
-        self.call_model('a_reset', models=self.models)
-        self.e_clear()
+        self.call_models('a_reset', models=self.models)
+        self.e_clear(models=self.models)
         self._p_restore()
+        self.is_setup = False
         self.setup()
 
     def add(self, model, param_dict=None, **kwargs):
@@ -213,6 +316,8 @@ class System(object):
         if model not in self.models:
             logger.warning(f"<{model}> is not an existing model.")
             return
+        if self.is_setup:
+            raise NotImplementedError("Adding devices are not allowed after setup.")
         group_name = self.__dict__[model].group
         group = self.groups[group_name]
 
@@ -239,42 +344,40 @@ class System(object):
             for fnd in mdl.services_fnd.values():
                 fnd.find_or_add(self)
 
-    def set_address(self, models=None):
+    def set_address(self, models):
         """
         Set addresses for differential and algebraic variables.
         """
-        if models is None:
-            models = self._models_flag['pflow']
-
         # set internal variable addresses
         for mdl in models.values():
-            if mdl.flags['address'] is True:
-                logger.debug(f'{mdl.class_name:10s}: addresses exist.')
+            if mdl.flags.address is True:
+                logger.debug(f'{mdl.class_name} address exists')
                 continue
             if mdl.n == 0:
                 continue
 
+            logger.debug(f'Setting address for {mdl.class_name}')
             n = mdl.n
             m0 = self.dae.m
             n0 = self.dae.n
             m_end = m0 + len(mdl.algebs) * n
             n_end = n0 + len(mdl.states) * n
-            collate = mdl.flags['collate']
+            collate = mdl.flags.collate
 
             if not collate:
                 for idx, item in enumerate(mdl.algebs.values()):
-                    item.set_address(np.arange(m0 + idx * n, m0 + (idx + 1) * n))
+                    item.set_address(np.arange(m0 + idx * n, m0 + (idx + 1) * n), contiguous=True)
                 for idx, item in enumerate(mdl.states.values()):
-                    item.set_address(np.arange(n0 + idx * n, n0 + (idx + 1) * n))
+                    item.set_address(np.arange(n0 + idx * n, n0 + (idx + 1) * n), contiguous=True)
             else:
                 for idx, item in enumerate(mdl.algebs.values()):
-                    item.set_address(np.arange(m0 + idx, m_end, len(mdl.algebs)))
+                    item.set_address(np.arange(m0 + idx, m_end, len(mdl.algebs)), contiguous=False)
                 for idx, item in enumerate(mdl.states.values()):
-                    item.set_address(np.arange(n0 + idx, n_end, len(mdl.states)))
+                    item.set_address(np.arange(n0 + idx, n_end, len(mdl.states)), contiguous=False)
 
             self.dae.m = m_end
             self.dae.n = n_end
-            mdl.flags['address'] = True
+            mdl.flags.address = True
 
         # set external variable addresses
         for mdl in models.values():
@@ -288,6 +391,12 @@ class System(object):
 
                 instance.link_external(ext_model)
 
+        # allocate memory for DAE arrays
+        self.dae.resize_arrays()
+
+        # set `v` and `e` in variables
+        self._set_var_arrays(models=models)
+
         # pre-allocate for names
         if len(self.dae.y_name) == 0:
             self.dae.x_name = [''] * self.dae.n
@@ -300,36 +409,63 @@ class System(object):
             self.dae.x_tex_name.extend([''] * (self.dae.n - len(self.dae.x_tex_name)))
             self.dae.y_tex_name.extend([''] * (self.dae.m - len(self.dae.y_tex_name)))
 
-    def set_dae_names(self, models=None):
+    def set_dae_names(self, models):
         """
         Set variable names for differential and algebraic variables, and discrete flags.
         """
-        if models is None:
-            models = self._models_flag['pflow']
+        def append_model_name(model_name, idx):
+            out = ''
+            if isinstance(idx, str):
+                if model_name in idx:
+                    out = idx
+            else:
+                out = f'{model_name} {idx}'
+
+            # replaces `_` with space for LaTeX to continue
+            out = out.replace('_', ' ')
+            return out
 
         for mdl in models.values():
             mdl_name = mdl.class_name
+            idx = mdl.idx
             for name, item in mdl.algebs.items():
-                for uid, addr in enumerate(item.a):
-                    self.dae.y_name[addr] = f'{mdl_name} {name} {uid}'
-                    self.dae.y_tex_name[addr] = rf'${item.tex_name}\ {mdl_name}\ {uid}$'
+                for id, addr in zip(idx.v, item.a):
+                    self.dae.y_name[addr] = f'{name} {append_model_name(mdl_name, id)}'
+                    self.dae.y_tex_name[addr] = rf'${item.tex_name}$ {append_model_name(mdl_name, id)}'
             for name, item in mdl.states.items():
-                for uid, addr in enumerate(item.a):
-                    self.dae.x_name[addr] = f'{mdl_name} {name} {uid}'
-                    self.dae.x_tex_name[addr] = rf'${item.tex_name}\ {mdl_name}\ {uid}$'
+                for id, addr in zip(idx.v, item.a):
+                    self.dae.x_name[addr] = f'{name} {append_model_name(mdl_name, id)}'
+                    self.dae.x_tex_name[addr] = rf'${item.tex_name}$ {append_model_name(mdl_name, id)}'
 
             # add discrete flag names
             if self.config.store_z == 1:
                 for item in mdl.discrete.values():
-                    if mdl.flags['initialized']:
+                    if mdl.flags.initialized:
                         continue
                     for name, tex_name in zip(item.get_names(), item.get_tex_names()):
-                        for uid in range(mdl.n):
-                            self.dae.z_name.append(f'{mdl_name} {name} {uid}')
-                            self.dae.z_tex_name.append(rf'${tex_name}\ {mdl_name}\ {uid}$')
+                        for id in idx.v:
+                            self.dae.z_name.append(f'{name} {append_model_name(mdl_name, id)}')
+                            self.dae.z_tex_name.append(rf'${item.tex_name}$ {append_model_name(mdl_name, id)}')
                             self.dae.o += 1
 
-    def init(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def _set_var_arrays(self, models):
+        """
+        Set arrays (`v` and `e`) in internal variables.
+
+        Parameters
+        ----------
+        models : OrderedDict, list, Model, optional
+            Models to execute.
+
+        """
+        for mdl in models.values():
+            if mdl.n == 0:
+                continue
+
+            for var in mdl.cache.vars_int.values():
+                var.set_arrays(self.dae)
+
+    def init(self, models: OrderedDict):
         """
         Initialize the variables for each of the specified models.
 
@@ -339,9 +475,6 @@ class System(object):
         - Call the model `init()` method, which initializes internal variables.
         - Copy variables to DAE and then back to the model.
         """
-        if models is None:
-            models = self._models_flag['pflow']
-
         for mdl in models.values():
             # link externals first
             for instance in mdl.services_ext.values():
@@ -356,38 +489,55 @@ class System(object):
             # initialize variables second
             mdl.init()
 
-            # TODO: re-think over the adder-setter approach and reduce data copy
-            self.vars_to_dae()
+            self.vars_to_dae(mdl)
             self.vars_to_models()
 
-        # store the inverse of time constants
-        self._store_Tf()
+        self.s_update_post(models)
 
-    def store_adder_setter(self, models=None):
+        # store the inverse of time constants
+        self._store_tf(models)
+
+    def store_adder_setter(self, models):
         """
-        Store the adders and setters for variables and equations.
+        Store non-inplace adders and setters for variables and equations.
         """
-        models = self._get_models(models)
         self._clear_adder_setter()
 
         for mdl in models.values():
+            # Note:
+            #   We assume that a Model with no device is not addressed and, therefore,
+            #   contains no value in each variable.
+            #   It is always true for the current architecture.
             if not mdl.n:
                 continue
-            for var in mdl.cache.all_vars.values():
-                if var.e_setter is False:
-                    self._adders[var.e_code].append(var)
-                else:
-                    self._setters[var.e_code].append(var)
 
-                if var.v_setter is False:
-                    self._adders[var.v_code].append(var)
-                else:
-                    self._setters[var.v_code].append(var)
+            # ``getters` that retrieve variable values from DAE
+            for var in mdl.cache.v_getters.values():
+                self._getters[var.v_code].append(var)
+
+            # ``adders`` that add variable values to the DAE array
+            for var in mdl.cache.v_adders.values():
+                self._adders[var.v_code].append(var)
+            for var in mdl.cache.e_adders.values():
+                self._adders[var.e_code].append(var)
+
+            # ``setters`` that force set variable values in the DAE array
+            for var in mdl.cache.v_setters.values():
+                self._setters[var.v_code].append(var)
+            for var in mdl.cache.e_setters.values():
+                self._setters[var.e_code].append(var)
+
+            # ``antiwindups`` stores all AntiWindup instances
             for item in mdl.discrete.values():
                 if isinstance(item, AntiWindup):
                     self.antiwindups.append(item)
 
+        return
+
     def link_ext_param(self, model=None):
+        """
+        Retrieve values for ``ExtParam`` for the given models.
+        """
         if model is None:
             models = self.models
         else:
@@ -418,7 +568,7 @@ class System(object):
             self.link_ext_param({mdl.class_name: mdl})
 
             # default Sn to Sb if not provided. Some controllers might not have Sn or Vn.
-            if 'Sn' in mdl.params:
+            if 'Sn' in mdl.__dict__:
                 Sn = mdl.Sn.v
             else:
                 Sn = Sb
@@ -426,26 +576,26 @@ class System(object):
             # If both Vn and Vn1 are not provided, default to Vn = Vb = 1
             # test if is shunt-connected or series-connected to bus, or unconnected to bus
             Vb, Vn = 1, 1
-            if 'bus' in mdl.params:
+            if 'bus' in mdl.__dict__:
                 Vb = self.Bus.get(src='Vn', idx=mdl.bus.v, attr='v')
-                Vn = mdl.Vn.v if 'Vn' in mdl.params else Vb
-            elif 'bus1' in mdl.params:
+                Vn = mdl.Vn.v if 'Vn' in mdl.__dict__ else Vb
+            elif 'bus1' in mdl.__dict__:
                 Vb = self.Bus.get(src='Vn', idx=mdl.bus1.v, attr='v')
-                Vn = mdl.Vn1.v if 'Vn1' in mdl.params else Vb
+                Vn = mdl.Vn1.v if 'Vn1' in mdl.__dict__ else Vb
 
             Zn = Vn ** 2 / Sn
             Zb = Vb ** 2 / Sb
 
             # process dc parameter pu conversion
             Vdcb, Vdcn, Idcn = 1, 1, 1
-            if 'node' in mdl.params:
+            if 'node' in mdl.__dict__:
                 Vdcb = self.Node.get(src='Vdcn', idx=mdl.node.v, attr='v')
-                Vdcn = mdl.Vdcn.v if 'Vdcn' in mdl.params else Vdcb
-                Idcn = mdl.Idcn.v if 'Idcn' in mdl.params else (Sb / Vdcb)
-            elif 'node1' in mdl.params:
+                Vdcn = mdl.Vdcn.v if 'Vdcn' in mdl.__dict__ else Vdcb
+                Idcn = mdl.Idcn.v if 'Idcn' in mdl.__dict__ else (Sb / Vdcb)
+            elif 'node1' in mdl.__dict__:
                 Vdcb = self.Node.get(src='Vdcn', idx=mdl.node1.v, attr='v')
-                Vdcn = mdl.Vdcn1.v if 'Vdcn1' in mdl.params else Vdcb
-                Idcn = mdl.Idcn.v if 'Idcn' in mdl.params else (Sb / Vdcb)
+                Vdcn = mdl.Vdcn1.v if 'Vdcn1' in mdl.__dict__ else Vdcb
+                Idcn = mdl.Idcn.v if 'Idcn' in mdl.__dict__ else (Sb / Vdcb)
             Idcb = Sb / Vdcb
             Rb = Vdcb / Idcb
             Rn = Vdcn / Idcn
@@ -466,31 +616,40 @@ class System(object):
                 for p in mdl.find_param(prop).values():
                     p.set_pu_coeff(coeff)
 
-    def l_update_var(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def l_update_var(self, models: OrderedDict):
         """
-        Update variable-based limiter discrete states.
+        Update variable-based limiter discrete states by calling ``l_update_var`` of models.
 
-        This function is usually called before any equation evaluation.
+        This function is must be called before any equation evaluation.
         """
-        self.call_model('l_update_var', models, self.dae.t)
+        self.call_models('l_update_var', models, self.dae.t)
 
-    def l_check_eq(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def l_update_eq(self, models:  OrderedDict):
         """
-        Update equation-dependent limiter discrete components.
+        First, update equation-dependent limiter discrete components by calling ``l_check_eq`` of models.
+        Second, force set equations after evaluating equations by calling ``l_set_eq`` of models.
 
-        This function is usually called after differential equation updates.
-        Currently, it is used exclusively for collecting anti-windup limiter status.
+        This function is must be called after differential equation updates.
         """
-        self.call_model('l_check_eq', models)
+        self.call_models('l_check_eq', models)
+        self.call_models('l_set_eq', models)
 
-    def l_set_eq(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def s_update_var(self, models: OrderedDict):
         """
-        Force set equations after evaluating equations.
+        Update variable services by calling ``s_update_var`` of models.
 
-        This function is evaluated afte ``l_check_eq``.
-        Currently, it is only used by anti-windup limiters to record changes.
+        This function is must be called before any equation evaluation after
+        limiter update function `l_update_var`.
         """
-        self.call_model('l_set_eq', models)
+        self.call_models('s_update_var', models)
+
+    def s_update_post(self, models: OrderedDict):
+        """
+        Update variable services by calling ``s_update_post`` of models.
+
+        This function is called at the end of `System.init()`.
+        """
+        self.call_models('s_update_post', models)
 
     def fg_to_dae(self):
         """
@@ -517,7 +676,7 @@ class System(object):
         Updated equation values remain in models and have not been collected into DAE at the end of this step.
         """
         try:
-            self.call_model('f_update', models)
+            self.call_models('f_update', models)
         except TypeError as e:
             logger.error("f_update failed. Did you forget to run `andes prepare -q` after updating?")
             raise e
@@ -531,12 +690,12 @@ class System(object):
         Like `f_update`, updated values have not collected into DAE at the end of the step.
         """
         try:
-            self.call_model('g_update', models)
+            self.call_models('g_update', models)
         except TypeError as e:
             logger.error("g_update failed. Did you forget to run `andes prepare -q` after updating?")
             raise e
 
-    def j_update(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def j_update(self, models: OrderedDict):
         """
         Call the Jacobian update method for models in sequence.
 
@@ -548,8 +707,7 @@ class System(object):
         -----
         Updated Jacobians are immediately reflected in the DAE sparse matrices (fx, fy, gx, gy).
         """
-        models = self._get_models(models)
-        self.call_model('j_update', models)
+        self.call_models('j_update', models)
 
         self.dae.restore_sparse()
         # collect sparse values into sparse structures
@@ -569,7 +727,7 @@ class System(object):
                                      f'j_size={j_size}')
                         raise e
 
-    def store_sparse_pattern(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def store_sparse_pattern(self, models: OrderedDict):
         """
         Collect and store the sparsity pattern of Jacobian matrices.
 
@@ -581,8 +739,7 @@ class System(object):
         It is a safeguard if the modeling user omitted the diagonal
         term in the equations.
         """
-        models = self._get_models(models)
-        self.call_model('store_sparse_pattern', models)
+        self.call_models('store_sparse_pattern', models)
 
         # add variable jacobian values
         for jname in jac_names:
@@ -612,34 +769,29 @@ class System(object):
             self.dae.store_sparse_ijv(jname, ii, jj, vv)
             self.dae.build_pattern(jname)
 
-    def vars_to_dae(self):
+    def vars_to_dae(self, model):
         """
         Copy variables values from models to `System.dae`.
 
         This function clears `DAE.x` and `DAE.y` and collects values from models.
-
-        Warnings
-        --------
-        Variables with property `v_setter=False` will be added to variable only if `v_str` is not None.
-        It prevents read-intended :py:mod:`andes.core.var.ExtVar` values to be summed.
         """
-        self.dae.clear_xy()
-        self._v_to_dae('x')
-        self._v_to_dae('y')
+        self._v_to_dae('x', model)
+        self._v_to_dae('y', model)
 
     def vars_to_models(self):
         """
         Copy variable values from `System.dae` to models.
         """
-        for var in self._adders['y'] + self._setters['y']:
+
+        for var in self._getters['y']:
             if var.n > 0:
                 var.v[:] = self.dae.y[var.a]
 
-        for var in self._adders['x'] + self._setters['x']:
+        for var in self._getters['x']:
             if var.n > 0:
                 var.v[:] = self.dae.x[var.a]
 
-    def _v_to_dae(self, v_name):
+    def _v_to_dae(self, v_code, model):
         """
         Helper function for collecting variable values into dae structures `x` and `y`.
 
@@ -648,29 +800,24 @@ class System(object):
 
         Parameters
         ----------
-        v_name : 'x' or 'y'
+        v_code : 'x' or 'y'
             Variable type name
         """
-        if v_name not in ('x', 'y'):
-            raise KeyError(f'{v_name} is not a valid var name')
+        if model.n == 0:
+            return
+        if model.flags.initialized is False:
+            return
 
-        for var in self._adders[v_name]:
-            # NOTE:
-            # For power flow, they will be initialized to zero.
-            # For TDS initialization, they will remain their value.
-            if var.n == 0:
+        for var in model.cache.v_adders.values():
+            if var.v_code != v_code:
                 continue
-            if isinstance(var, ExtVar) and (var.v_str is None):
-                continue
-            if var.owner.flags['initialized'] is False:
-                continue
-            np.add.at(self.dae.__dict__[v_name], var.a, var.v)
+            np.add.at(self.dae.__dict__[v_code], var.a, var.v)
 
-        for var in self._setters[v_name]:
-            if var.owner.flags['initialized'] is False:
+        for var in self._setters[v_code]:
+            if var.owner.flags.initialized is False:
                 continue
             if var.n > 0:
-                np.put(self.dae.__dict__[v_name], var.a, var.v)
+                np.put(self.dae.__dict__[v_code], var.a, var.v)
 
     def _e_to_dae(self, eq_name: str):
         """
@@ -681,15 +828,10 @@ class System(object):
         eq_name : 'x' or 'y'
             Equation type name
         """
-        if eq_name not in ('f', 'g'):
-            raise KeyError(f'{eq_name} is not a valid eq name')
-
         for var in self._adders[eq_name]:
-            if var.n > 0:
-                np.add.at(self.dae.__dict__[eq_name], var.a, var.e)
+            np.add.at(self.dae.__dict__[eq_name], var.a, var.e)
         for var in self._setters[eq_name]:
-            if var.n > 0:
-                np.put(self.dae.__dict__[eq_name], var.a, var.e)
+            np.put(self.dae.__dict__[eq_name], var.a, var.e)
 
     def get_z(self, models: Optional[Union[str, List, OrderedDict]] = None):
         """
@@ -710,15 +852,23 @@ class System(object):
 
         return np.concatenate(z_dict)
 
-    def find_models(self, flag: Optional[Union[str, Tuple]] = None):
+    def find_models(self, flag: Optional[Union[str, Tuple]], skip_zero: bool = True):
         """
-        Find models whose ``flag`` field is True.
+        Find models with at least one of the flags as True.
+
+        Warnings
+        --------
+        Checking the number of devices has been centralized into this function.
+        ``models`` passed to most System calls must be retrieved from here.
 
         Parameters
         ----------
         flag : list, str
             Flags to find
 
+        skip_zero : bool
+            Skip models with zero devices
+
         Returns
         -------
         OrderedDict
@@ -730,8 +880,13 @@ class System(object):
 
         out = OrderedDict()
         for name, mdl in self.models.items():
+
+            if skip_zero is True:
+                if (mdl.n == 0) or (mdl.in_use is False):
+                    continue
+
             for f in flag:
-                if mdl.flags[f] is True:
+                if mdl.flags.__dict__[f] is True:
                     out[name] = mdl
                     break
         return out
@@ -755,6 +910,19 @@ class System(object):
         with open(pkl_path, 'wb') as f:
             dill.dump(self.calls, f)
 
+    @staticmethod
+    def _load_pkl():
+        import dill
+        dill.settings['recurse'] = True
+        pkl_path = get_pkl_path()
+
+        if os.path.isfile(pkl_path):
+            with open(pkl_path, 'rb') as f:
+                loaded_calls = dill.load(f)
+                return loaded_calls
+        else:
+            return None
+
     def undill(self):
         """
         Deserialize the function calls from ``~/andes.calls.pkl`` with dill.
@@ -762,17 +930,22 @@ class System(object):
         If no change is made to models, future calls to ``prepare()`` can be replaced with ``undill()`` for
         acceleration.
         """
-        import dill
-        dill.settings['recurse'] = True
 
-        pkl_path = get_pkl_path()
-        if not os.path.isfile(pkl_path):
+        loaded_calls = self._load_pkl()
+
+        if loaded_calls is not None:
+            ver = loaded_calls.get('__version__')
+            if ver == __version__:
+                self.calls = loaded_calls
+                logger.debug(f'Undilled calls from "{get_pkl_path()}" is current.')
+            else:
+                logger.info(f'Undilled calls are for version {ver}, regenerating...')
+                self.prepare(quick=True, incremental=True)
+
+        else:
+            logger.info('Generating numerical calls at the first launch.')
             self.prepare()
 
-        with open(pkl_path, 'rb') as f:
-            self.calls = dill.load(f)
-        logger.debug(f'Undill loaded "{pkl_path}" file.')
-
         for name, model_call in self.calls.items():
             if name in self.__dict__:
                 self.__dict__[name].calls = model_call
@@ -784,7 +957,7 @@ class System(object):
         The output is an OrderedDict of model names and instances.
         """
         if models is None:
-            models = self._models_flag['pflow']
+            models = self.exist.pflow
         if isinstance(models, str):
             models = {models: self.__dict__[models]}
         elif isinstance(models, Model):
@@ -801,18 +974,16 @@ class System(object):
         # do nothing for OrderedDict type
         return models
 
-    def _store_Tf(self):
+    def _store_tf(self, models):
         """
-        Store the inverse time constant associated with equations
+        Store the inverse time constant associated with equations.
         """
-        for var in self._adders['f']:
-            if var.t_const is not None:
-                np.put(self.dae.Tf, var.a, var.t_const.v)
-        for var in self._setters['f']:
-            if var.t_const is not None:
-                np.put(self.dae.Tf, var.a, var.t_const.v)
+        for mdl in models.values():
+            for var in mdl.cache.states_and_ext.values():
+                if var.t_const is not None:
+                    np.put(self.dae.Tf, var.a, var.t_const.v)
 
-    def call_model(self, method: str, models: Optional[Union[str, list, Model, OrderedDict]], *args, **kwargs):
+    def call_models(self, method: str, models: OrderedDict, *args, **kwargs):
         """
         Call methods on the given models.
 
@@ -832,8 +1003,6 @@ class System(object):
         The return value of the models in an OrderedDict
 
         """
-        models = self._get_models(models)
-
         ret = OrderedDict()
         for name, mdl in models.items():
             ret[name] = getattr(mdl, method)(*args, **kwargs)
@@ -893,26 +1062,8 @@ class System(object):
                         uid = dest_model.idx2uid(dest_idx)
                         dest_model.services_ref[n].v[uid].append(model_idx)
 
-    def _generate_pycode_file(self):
-        """
-        Generate empty files for storing lambdified Python code (TODO)
-        """
-        self.call_model('generate_pycode_file', self.models)
-
-    def _generate_initializers(self):
-        self.call_model('generate_initializers', self.models)
-
-    def _generate_symbols(self):
-        self.call_model('generate_symbols', self.models)
-
-    def _generate_pretty_print(self):
-        self.call_model('generate_pretty_print', self.models)
-
-    def _generate_equations(self):
-        self.call_model('generate_equations', self.models)
-
-    def _generate_jacobians(self):
-        self.call_model('generate_jacobians', self.models)
+            # set model ``in_use`` flag
+            model.set_in_use()
 
     def import_groups(self):
         """
@@ -985,8 +1136,9 @@ class System(object):
                 attr_name = cls_name
                 self.__dict__[attr_name] = the_class(system=self, config=self._config_object)
                 self.routines[attr_name] = self.__dict__[attr_name]
+                self.routines[attr_name].config.check()
 
-    def store_switch_times(self, models=None):
+    def store_switch_times(self, models):
         """
         Store event switching time in a sorted Numpy array at ``System.switch_times``.
 
@@ -995,24 +1147,24 @@ class System(object):
         array-like
             self.switch_times
         """
-        models = self._get_models(models)
         out = []
         for instance in models.values():
             out.extend(instance.get_times())
 
         out = np.ravel(np.array(out))
+        out = np.append(out, out + 1e-4)
         out = np.unique(out)
         out = out[np.where(out >= 0)]
         out = np.sort(out)
 
         self.switch_times = out
+        self.n_switches = len(self.switch_times)
         return self.switch_times
 
-    def switch_action(self, models=None):
+    def switch_action(self, models):
         """
         Invoke the actions associated with switch times.
         """
-        models = self._get_models(models)
         for instance in models.values():
             instance.switch_action(self.dae.t)
 
@@ -1024,32 +1176,35 @@ class System(object):
             for param in model.num_params.values():
                 param.restore()
 
-    def e_clear(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def e_clear(self, models: OrderedDict):
         """
         Clear equation arrays in DAE and model variables.
 
         This step must be called before calling `f_update` or `g_update` to flush existing values.
         """
         self.dae.clear_fg()
-        self.call_model('e_clear', models)
+        self.call_models('e_clear', models)
 
     def remove_pycapsule(self):
         """
         Remove PyCapsule objects in solvers.
         """
         for r in self.routines.values():
-            r.solver.remove_pycapsule()
+            r.solver.clear()
 
     def _store_calls(self):
         """
         Collect and store model calls into system.
         """
         logger.debug("Collecting Model.calls into System.")
+
+        self.calls['__version__'] = __version__
+
         for name, mdl in self.models.items():
             self.calls[name] = mdl.calls
 
     def _list2array(self):
-        self.call_model('list2array', self.models)
+        self.call_models('list2array', self.models)
 
     def set_config(self, config=None):
         """
@@ -1104,7 +1259,7 @@ class System(object):
 
         conf = configparser.ConfigParser()
         conf.read(conf_path)
-        logger.debug(f'Config loaded from file "{conf_path}".')
+        logger.info(f'Loaded config from file "{conf_path}"')
         return conf
 
     def save_config(self, file_path=None, overwrite=False):
@@ -1140,7 +1295,7 @@ class System(object):
         logger.info(f'Config written to "{file_path}"')
         return file_path
 
-    def supported_models(self):
+    def supported_models(self, export='plain'):
         """
         Return the support group names and model names in a table.
 
@@ -1161,6 +1316,7 @@ class System(object):
         tab = Tab(title='Supported Groups and Models',
                   header=['Group', 'Models'],
                   data=pairs,
+                  export=export,
                   )
 
         return tab.draw()
