@@ -3,10 +3,11 @@ import andes
 
 
 class TestCodegen(unittest.TestCase):
-    def setUp(self) -> None:
+    def test_1_docs(self) -> None:
         self.ss = andes.main.prepare(quick=True)
-
-    def test_docs(self) -> None:
         out = ''
         for group in self.ss.groups.values():
             out += group.doc_all()
+
+    def test_codegen_inc(self):
+        self.ss = andes.main.prepare(quick=True, incremental=True)
